﻿Imports System.Runtime.CompilerServices

Public Module Program

   Private _Klamotten As New DataSet

   ''' <summary>
   ''' Falls tableName nicht im Dataset enthalten, erzeuge diese DataTable
   ''' und statte sie mit einer Primärschlüsselspalte namens 'ID' aus
   ''' </summary>
   Private Function GetOrCreateTable(ByVal tableName As String) As DataTable
      If _Klamotten.Tables.Contains(tableName) Then Return _Klamotten.Tables(tableName)
      Dim tb = _Klamotten.Tables.Add(tableName)
      tb.PrimaryKey = {tb.Columns.Add("ID")}
      Return tb
   End Function

   ''' <summary>
   ''' Erzeuge die ForeignKey-spalte 'under.upperTableNameID',
   ''' und dann eine DataRelation von upper.PrimaryKey zum neu erstellten under.ForeignKey
   ''' </summary>
   Private Function CreateRelation(ByVal upper As DataTable, ByVal under As DataTable) _
         As DataRelation
      Dim colUnder = under.Columns.Add(upper.TableName & "ID")
      Dim relName = upper.TableName & "_" & under.TableName
      Return _Klamotten.Relations.Add(relName, upper.PrimaryKey(0), colUnder, True)
   End Function

   ''' <summary>
   ''' ermittle oder erzeuge die in names aufgeführten DataTables
   ''' </summary>
   Private Function Erst(ByVal ParamArray names() As String) As DataTable()
      Return names.Select(Function(name) GetOrCreateTable(name)).ToArray
   End Function

   ''' <summary>
   ''' ordne jeder der DataTables in tbls alle in names genannten DataTables unter.
   ''' returne die untergeordneten DataTables
   ''' </summary>
   <Extension()> _
   Private Function Danach(ByVal tbls As IEnumerable(Of DataTable), _
         ByVal ParamArray names() As String) As HashSet(Of DataTable)
      Danach = New HashSet(Of DataTable)
      For Each upper In tbls
         For Each under In From name In names Select GetOrCreateTable(name)
            Danach.Add(CreateRelation(upper, under).ChildTable)
         Next
      Next
   End Function

   Public Sub CreateTopoOrder()
      'topologische Rangordnung per Code definieren
      Erst("Unterhemd").Danach("Pullover").Danach("Mantel")
      Erst("Unterhose").Danach("Hose").Danach("LinkerSchuh", "RechterSchuh", "Mantel") _
         .Danach("RechterHandschuh", "LinkerHandschuh")
      Erst("Socke1").Danach("LinkerSchuh")
      Erst("Socke2").Danach("RechterSchuh")
   End Sub

   Public Sub Main(ByVal args As String())
      Console.WriteLine("codeseitig erstelltes Dataset sortieren:")
      CreateTopoOrder()
      _Klamotten.GetRankedTables.ForEach(Sub(tb As DataTable) Console.WriteLine(tb.TableName))
      Console.WriteLine("")
      Console.WriteLine("im Designer gefertigtes Dataset:")
      _Klamotten = New KlamottenDts
      _Klamotten.GetRankedTables.ForEach(Sub(tb As DataTable) Console.WriteLine(tb.TableName))
      Console.ReadKey()
   End Sub

End Module
